# Встановлення Django

> Частина цього розділу базується на матеріалах Geek Girls Carrots (http://django.carrots.pl/).
> 
> Частина цього параграфу базується на матеріалі довідника [django-marcador tutorial][1] ліцензованого Creative Commons Attribution-ShareAlike 4.0 International License. Авторське право на навчальні матеріали django-marcador tutorial належить Markus Zapke-Gründemann та ін.

 [1]: http://django-marcador.keimlink.de/

## Віртуальне середовище

Перед тим як встановлювати Django, ми допоможемо вам встановити надзвичайно корисний інструмент, що допоможе підтримувати чистоту середовища з кодом на вашому комп'ютері. Можете пропустити цей крок, але дуже бажано не робити цього, адже стартування із найліпшим чином налаштованою системою вбереже вас від багатьох проблем в майбутньому!

Отже, створимо віртуальне середовище (англ. **virtual environment** або скорочено *virtualenv*). Це ізолює ваші налаштування для Python/Django для кожного окремого проекту. Іншими словами, будь-які зміни здійснені на одному сайті жодним чином не вплинуть на інші розроблювані вами сайти. Гарно, правда ж?

Все що вам необхідно зробити це знайти місце, де ви хочете створити віртуальне середовище `virtualenv`; наприклад, ваша домашня папка. На Windows це може виглядати як `C:\Users\Name` (де `Name` - ваш логін).

В рамках цього навчального посібника будемо використовувати нову директорію `djangogirls` з вашої домашньої папки:

    mkdir djangogirls
    cd djangogirls
    

Створимо віртуальне середовище з ім'ям `myvenv`. Загальна команда буде ось в такому форматі:

    python -m venv myvenv
    

### Windows

Щоб створити нове віртуальне середовище `virtualenv`, вам потрібно відкрити консоль (ми вже розповідали вам про це в одному з попередніх розділів - пам'ятаєте?) і запустити `C:\Python34\python -m venv myvenv`. Це буде виглядати так:

    C:\Users\Name\djangogirls> C:\Python34\python -m venv myvenv
    

де `C:\Python34\python` - це папка, в якій ви перед цим встановили Python, а `myvenv` - ім'я вашого віртуального середовища `virtualenv`. Можете використовувати яке завгодно ім'я, але воно має містити лише маленькі літери і не містити пробілів. Також хороша ідея зберігати ім'я в короткому виді, оскільки ви будете багаторазово посилатися на нього!

### Linux та OS X

Створення віртуального середовища `virtualenv` як на Linux так і на OS X просто відбувається із запуском `python3 -m venv myvenv`. Виглядає це так:

    ~/djangogirls$ python3 -m venv myvenv
    

`myvenv` - ім'я вашого віртуального середовища `virtualenv`. Можете використовувати яке завгодно ім'я, але воно має містити лише маленькі літери і не містити пробілів. Також хороша ідея зберігати ім'я в короткому виді, оскільки ви будете багаторазово посилатися на нього!

> **ЗАУВАЖЕННЯ:** Ініціалізація віртуального середовища на Ubuntu 14.04 схожим методом наразі призводить до наступної помилки:
> 
>     Error: Command '['/home/eddie/Slask/tmp/venv/bin/python3', '-Im', 'ensurepip', '--upgrade', '--default-pip']' returned non-zero exit status 1
>     
> 
> Щоб обійти цю проблему, використовуйте натомість команду `virtualenv`.
> 
>     ~/djangogirls$ sudo apt-get install python-virtualenv
>     ~/djangogirls$ virtualenv --python=python3.4 myvenv
>     

## Робота з віртуальним середовищем

Вищезазначена команда створить папку `myvenv` (або інше вибране вами ім'я), що міститиме наше віртуальне середовище (в основному, набір папок і файлів). Все, що ми хочемо тепер зробити це активувати середовище шляхом запуску:

    C:\Users\Name\djangogirls> myvenv\Scripts\activate
    

для Windows, або:

    ~/djangogirls$ source myvenv/bin/activate
    

для OS X і Linux.

Не забудьте замість `myvenv` повинно бути вибране вами ім'я для віртуального середовища!

> **ЗАУВАЖЕННЯ:** іноді використання `source` може бути недоступно. В цьому випадку спробуйте зробити наступне: 
> 
>     ~/djangogirls$ . myvenv/bin/activate
>     

Про активізацію віртуального середовища ви дізнаєтесь коли побачите підказку в командному рядку консолі, котра виглядатиме наступним чином:

    (myvenv) C:\Users\Name\djangogirls>
    

або:

    (myvenv) ~/djangogirls$
    

Зауважте з'явився префікс `(myvenv)`!

Під час роботи у віртуальному середовищі, `python` буде автоматичному посилатися на правильну версію, отже ви можете використовувати `python` замість `python3`.

Добре, маємо усі необхідні залежності. Можемо зрештою встановити Django!

## Встановлення Django

Наразі, коли ваше віртуальне середовище активоване, можна встановлювати Django використавши `pip`. В консолі, запустіть `pip install django==1.8` (зазначте, що тут ми користуємося подвійним знаком рівності: `==`).

    (myvenv) ~$ pip install django==1.8
    Downloading/unpacking django==1.8
    Installing collected packages: django
    Successfully installed django
    Cleaning up...


для Windows

> Якщо виникла помилка під час запуску pip на Windows платформі, будь ласка, перевірте чи шлях до вашого проекту містить пробіли (наприклад, `C:\Users\User Name\djangogirls`). Якщо так, будь ласка, перемістіть проект в інше місце, шлях до якого не буде містити пробілів (наприклад: `C:\djangogirls`). Після переміщення спробуйте запустити вищевказану команду знову.

для Linux

> Якщо виникла помилка під час запуску pip на Ubuntu 12.04, будь ласка, запустіть `python -m pip install -U --force-reinstall pip` щоб коректно перевстановити pip у віртуальному середовищі.

Ось і все! Тепер (нарешті) ви готові до створення Django додатку!
